"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var view_1 = require("tns-core-modules/ui/core/view");
var enums_1 = require("tns-core-modules/ui/enums");
var grid_layout_1 = require("tns-core-modules/ui/layouts/grid-layout");
var platform_1 = require("tns-core-modules/platform");
var enums = require("tns-core-modules/ui/enums");
var builder = require("tns-core-modules/ui/builder");
var unfilteredSource = [];
var filtering = false;
exports.listWidthProperty = new view_1.Property({
    name: "listWidth",
    defaultValue: "300"
});
exports.listHeightProperty = new view_1.Property({
    name: "listHeight",
    defaultValue: "300"
});
exports.headingTitleProperty = new view_1.Property({
    name: "headingTitle",
    defaultValue: undefined
});
exports.enableSearchProperty = new view_1.Property({
    name: "enableSearch",
    defaultValue: true,
    valueConverter: view_1.booleanConverter
});
exports.showCancelProperty = new view_1.Property({
    name: "showCancel",
    defaultValue: true,
    valueConverter: view_1.booleanConverter
});
exports.dimmerColorProperty = new view_1.Property({
    name: "dimmerColor",
    defaultValue: "rgba(0,0,0,0.8)"
});
exports.blurProperty = new view_1.Property({
    name: "blur",
    defaultValue: "none"
});
exports.focusOnShowProperty = new view_1.Property({
    name: "focusOnShow",
    defaultValue: false
});
exports.hideFilterProperty = new view_1.Property({
    name: "hideFilter",
    defaultValue: false
});
exports.hintTextProperty = new view_1.Property({
    name: "hintText",
    defaultValue: "Enter text to filter..."
});
exports.sourceProperty = new view_1.Property({
    name: "source",
    defaultValue: undefined,
    affectsLayout: true,
    valueChanged: function (target, oldValue, newValue) {
        if (!filtering) {
            while (unfilteredSource.length)
                unfilteredSource.pop();
            newValue.forEach(function (element) {
                unfilteredSource.push(element);
            });
        }
    }
});
var FilterableListpicker = (function (_super) {
    __extends(FilterableListpicker, _super);
    function FilterableListpicker() {
        var _this = _super.call(this) || this;
        _this.blurView = false;
        _this._isAutocomplete = false;
        _this.visibility = enums.Visibility.collapse;
        _this._searchFilter = _this._searchFilterFn.bind(_this);
        return _this;
    }
    FilterableListpicker.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        var innerComponent = builder.parse("\n          <GridLayout id=\"dc_flp_container\" class=\"flp-container\" visibility=\"collapsed\" loaded=\"{{loadedContainer}}\">\n              <StackLayout tap=\"{{cancel}}\" width=\"100%\" height=\"100%\"></StackLayout>\n              <GridLayout width=\"{{listWidth}}\" verticalAlignment=\"middle\" rows=\"auto, auto, auto, auto\" id=\"dc_flp\" class=\"flp-list-container\" loaded=\"{{loadedInnerContainer}}\">\n                  <Label row=\"0\" text=\"{{headingTitle ? headingTitle : ''}}\" class=\"flp-heading-title\" visibility=\"{{headingTitle ? 'visible' : 'collapsed'}}\"></Label>\n                  <TextField hint=\"{{hintText}}\" row=\"1\" text=\"{{filterText}}\" id=\"filterTextField\" class=\"flp-hint-field\" visibility=\"{{enableSearch ? 'visible' : 'collapsed'}}\" loaded=\"{{loadedTextField}}\"></TextField>\n                  <ListView items=\"{{ source }}\" row=\"2\" height=\"{{listHeight}}\" itemTap=\"{{choose}}\" class=\"flp-listview\">\n                      <ListView.itemTemplate>\n                          <StackLayout class=\"flp-row\">\n                              <GridLayout columns=\"auto, *, auto\" visibility=\"{{title ? 'visible' : 'collapsed'}}\" class=\"flp-row-container\">\n                                  <Image src=\"{{image ? image : null}}\" width=\"30\" visibility=\"{{image ? 'visible' : 'collapsed'}}\" stretch=\"aspectFit\" rowSpan=\"2\" class=\"flp-image\"></Image>\n                                  <StackLayout class=\"flp-title-container\" col=\"1\" verticalAlignment=\"middle\">\n                                      <Label text=\"{{title ? title : ''}}\" textWrap=\"true\" class=\"flp-title\"></Label>\n                                      <Label text=\"{{description ? description : ''}}\" textWrap=\"true\" visibility=\"{{description ? 'visible' : 'collapsed'}}\" class=\"flp-description\"></Label>\n                                  </StackLayout>\n                                  <Label col=\"2\" text=\"{{selected ? selected : ''}}\" class=\"flp-item-selected\" visibility=\"{{selected ? 'visible' : 'collapsed'}}\"></Label>\n                              </GridLayout>\n                              <Label text=\"{{$value}}\" textWrap=\"true\" class=\"flp-no-title\" visibility=\"{{title ? 'collapsed' : 'visible'}}\"></Label>\n                          </StackLayout>\n                      </ListView.itemTemplate>\n                  </ListView>\n                  <StackLayout row=\"3\" class=\"flp-cancel-container\" visibility=\"{{showCancel ? 'visible' : 'collapsed'}}\">\n                      <Button text=\"Cancel\" tap=\"{{cancel}}\" verticalAlignment=\"middle\" class=\"flp-btn-cancel\"></Button>\n                  </StackLayout>\n              </GridLayout>\n          </GridLayout>");
        innerComponent.bindingContext = this;
        this.addChild(innerComponent);
    };
    FilterableListpicker.prototype.loadedContainer = function (args) {
        this._container = args.object;
    };
    FilterableListpicker.prototype.loadedInnerContainer = function (args) {
        this._picker = args.object;
    };
    FilterableListpicker.prototype.autocomplete = function (fn) {
        var _this = this;
        if (!this.isAutocomplete)
            return;
        if (typeof fn !== "function")
            throw ("[FilterableListPicker]: autotcomplete params must be a Function type !");
        if (!this.source)
            this.source = [];
        this._suggestions = this.source;
        this._textField.on("textChange", function (data) {
            if (!data.value && _this._suggestions.length > 0) {
                _this.set("source", _this._suggestions);
                _this.notifyPropertyChange("source", _this._suggestions);
                console.log(_this._suggestions);
                return;
            }
            fn(data);
        });
    };
    FilterableListpicker.prototype.loadedTextField = function (args) {
        this._textField = args.object;
    };
    FilterableListpicker.prototype.choose = function (args) {
        var item = this.source[args.index];
        this.hide();
        this.notify({
            eventName: "itemTapped",
            object: this,
            selectedItem: item
        });
    };
    FilterableListpicker.prototype.cancel = function () {
        this.notify({
            eventName: "canceled",
            object: this
        });
        this.hide();
    };
    FilterableListpicker.prototype.hide = function () {
        var _this = this;
        if (this.enableSearch) {
            if (this._textField.dismissSoftInput)
                this._textField.dismissSoftInput();
            this._textField.text = "";
        }
        if (this.blurView) {
            UIView.animateWithDurationAnimationsCompletion(0.3, function () {
                _this.blurView.effect = null;
            }, function () {
                _this.blurView.removeFromSuperview();
            });
        }
        else {
            this._container
                .animate({
                opacity: 0,
                duration: 200
            })
                .then(function (_) { }, function (err) { });
        }
        if (this.enableSearch) {
            this._textField.off("textChange", this._searchFilter);
        }
        return this._picker
            .animate({
            scale: { x: 0.7, y: 0.7 },
            opacity: 0,
            duration: 400,
            curve: enums_1.AnimationCurve.cubicBezier(0.1, 0.1, 0.1, 1)
        })
            .then(function () {
            _this.visibility = enums.Visibility.collapse;
            _this._container.visibility = "collapse";
        }, function (err) { });
    };
    FilterableListpicker.prototype.show = function () {
        var _this = this;
        this.visibility = enums.Visibility.visible;
        this._container.visibility = "visible";
        this.source = unfilteredSource.filter(function (i) { return true; });
        if (platform_1.isIOS && this.blur && this.blur != "none") {
            var iosView = this._container.ios;
            var effectView_1 = UIVisualEffectView.alloc().init();
            effectView_1.frame = CGRectMake(0, 0, iosView.bounds.size.width, iosView.bounds.size.height);
            effectView_1.autoresizingMask =
                2 | 16;
            this.blurView = effectView_1;
            iosView.addSubview(effectView_1);
            iosView.sendSubviewToBack(effectView_1);
            UIView.animateWithDurationAnimationsCompletion(0.3, function () {
                var theme = 2;
                if (_this.blur == "light")
                    theme = 1;
                effectView_1.effect = UIBlurEffect.effectWithStyle(theme);
            }, function () {
            });
        }
        else {
            this._container.opacity = 0;
            this._container.backgroundColor = this.dimmerColor;
            this._container
                .animate({
                opacity: 1,
                duration: 200
            })
                .then(function (_) { }, function (err) { });
        }
        this._picker.scaleX = 0.7;
        this._picker.scaleY = 0.7;
        this._picker.opacity = 0;
        this._picker
            .animate({
            scale: { x: 1, y: 1 },
            opacity: 1,
            duration: 400,
            curve: enums_1.AnimationCurve.cubicBezier(0.1, 0.1, 0.1, 1)
        })
            .then(function (_) { }, function (err) { });
        if (this.enableSearch) {
            if (JSON.parse(this.focusOnShow))
                this._textField.focus();
            this._textField.on("textChange", this._searchFilter);
        }
    };
    Object.defineProperty(FilterableListpicker.prototype, "isAutocomplete", {
        get: function () {
            return this._isAutocomplete;
        },
        set: function (val) {
            if (val === false) {
                console.log(val);
                this._textField.off("textChange");
            }
            this._isAutocomplete = val;
        },
        enumerable: true,
        configurable: true
    });
    FilterableListpicker.prototype._searchFilterFn = function (data) {
        filtering = true;
        this.source = unfilteredSource.filter(function (item) {
            if (item.title) {
                return (item.title.toLowerCase().indexOf(data.value.toLowerCase()) !== -1);
            }
            else {
                return item.toLowerCase().indexOf(data.value.toLowerCase()) !== -1;
            }
        });
        filtering = false;
    };
    FilterableListpicker.canceledEvent = "canceled";
    FilterableListpicker.itemTappedEvent = "itemTapped";
    return FilterableListpicker;
}(grid_layout_1.GridLayout));
exports.FilterableListpicker = FilterableListpicker;
exports.listWidthProperty.register(FilterableListpicker);
exports.listHeightProperty.register(FilterableListpicker);
exports.headingTitleProperty.register(FilterableListpicker);
exports.enableSearchProperty.register(FilterableListpicker);
exports.showCancelProperty.register(FilterableListpicker);
exports.dimmerColorProperty.register(FilterableListpicker);
exports.focusOnShowProperty.register(FilterableListpicker);
exports.hideFilterProperty.register(FilterableListpicker);
exports.blurProperty.register(FilterableListpicker);
exports.hintTextProperty.register(FilterableListpicker);
exports.sourceProperty.register(FilterableListpicker);
//# sourceMappingURL=filterable-listpicker.js.map